package com.ruoyi.web.controller.brewery;

import java.util.List;
import javax.servlet.http.HttpServletResponse;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.brewery.domain.WyRedeemOrder;
import com.ruoyi.brewery.service.IWyRedeemOrderService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 兑换订单Controller
 * 
 * @author ruoyi
 * @date 2024-12-21
 */
@Api(tags = "兑换订单")
@RestController
@RequestMapping("/system/brewery/redeemOrder")
public class WyRedeemOrderController extends BaseController
{
    @Autowired
    private IWyRedeemOrderService wyRedeemOrderService;

    /**
     * 查询兑换订单列表
     */
    @ApiOperation("查询兑换订单列表")
    @PreAuthorize("@ss.hasPermi('brewery:redeemOrder:list')")
    @GetMapping("/list")
    public TableDataInfo list(WyRedeemOrder wyRedeemOrder)
    {
        startPage();
        List<WyRedeemOrder> list = wyRedeemOrderService.selectWyRedeemOrderList(wyRedeemOrder);
        return getDataTable(list);
    }

    /**
     * 导出兑换订单列表
     */
    @ApiOperation("导出兑换订单列表")
    @PreAuthorize("@ss.hasPermi('brewery:redeemOrder:export')")
    @Log(title = "兑换订单", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, WyRedeemOrder wyRedeemOrder)
    {
        List<WyRedeemOrder> list = wyRedeemOrderService.selectWyRedeemOrderList(wyRedeemOrder);
        ExcelUtil<WyRedeemOrder> util = new ExcelUtil<WyRedeemOrder>(WyRedeemOrder.class);
        util.exportExcel(response, list, "兑换订单数据");
    }

    /**
     * 获取兑换订单详细信息
     */
    @ApiOperation("获取兑换订单详细信息")
    @PreAuthorize("@ss.hasPermi('brewery:redeemOrder:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(wyRedeemOrderService.selectWyRedeemOrderById(id));
    }

    /**
     * 新增兑换订单
     */
    @ApiOperation("新增兑换订单")
    @PreAuthorize("@ss.hasPermi('brewery:redeemOrder:add')")
    @Log(title = "兑换订单", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody WyRedeemOrder wyRedeemOrder)
    {
        return toAjax(wyRedeemOrderService.insertWyRedeemOrder(wyRedeemOrder));
    }

    /**
     * 修改兑换订单
     */
    @ApiOperation("修改兑换订单")
    @PreAuthorize("@ss.hasPermi('brewery:redeemOrder:edit')")
    @Log(title = "兑换订单", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody WyRedeemOrder wyRedeemOrder)
    {
        return toAjax(wyRedeemOrderService.updateWyRedeemOrder(wyRedeemOrder));
    }

    /**
     * 删除兑换订单
     */
    @ApiOperation("删除兑换订单")
    @PreAuthorize("@ss.hasPermi('brewery:redeemOrder:remove')")
    @Log(title = "兑换订单", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(wyRedeemOrderService.deleteWyRedeemOrderByIds(ids));
    }
}
